//Platformer Movement
atom
	var //We include these variables for both movable and immoble atoms, in case we want areas or turf to bestow their atributes on a mob or object.
		jump_max = 1			//Maximum number of times mob may jump prior to landing. Increase to allow double or triple jumps. 0 for no max.
		jump_speed_max = 7.5	//Number of pixels to move on the first tick of the jump. (If greater than world.icon_size, collisions may have bugs.)
		gravity = 0.55			//Change in speed for each tick of the jump.
		freefall = -30			//Maximum fall speed. (If greater than world.icon_size, collisions from long falls may have bugs.)
	movable
		var
			jump = 0		//Count number of jumps from last time ground was touched.
			jump_speed = 0	//Number of pixels to move per tick of the jump. Decreases by gravity each tick.
			jump_speed_peaked = 0	//Keep track of whether atom has surpassed the peak of its jump, yet.
		proc
			Jump(var/start_speed = 0)
				if((jump < jump_max) || (jump_max <= 0) || (DensityBelow() == -1)) //If mob has not reached their jump max, allow them to jump.
					jump += 1
					jump_speed = start_speed
					jump_speed_peaked = 0
					Jumped(start_speed)
					JumpCycle(jump)
			Jumped(var/start_speed = 0)
				if(start_speed > 0 && client) range() << soundJump()
			JumpCycle(var/j, var/cycle = 0)
				if(j == jump) //Make sure each mob only has one JumpCycle at a time. This prevents multi jumps from having multiple JumpCycle procs running.
					var
						d = NORTH
						dy = step_y+jump_speed
					if(dy <= 0) d = SOUTH
					Move(loc, d, step_x, dy)
					if(!jump_speed_peaked)
						if(client)
							if(client.north)
								jump_speed += gravity
							else
								jump_speed_peaked = 1
						else
							jump_speed += gravity
					if(jump_speed_peaked || (client && !client.north)) jump_speed -= gravity
					if(jump_speed > jump_speed_max)
						jump_speed = jump_speed_max	//Max upward speed
						jump_speed_peaked = 1
					if(jump_speed < freefall) jump_speed = freefall				//Max downward speed
					if(j == jump) spawn(1/world.fps) JumpCycle(j, cycle+1) //Only spawn next cycle under conditions that prevent bug of multiple jump cycles running at same time.
			DensityBelow() //Count how many dense objects are directly (1 pixel) below the src, and return that count.
				var/c = -1 //Counts the number of dense atoms found.  -1 if no atoms found (i.e. the search was at the map's edge).
				for(var/atom/A in obounds(src, 0, -1, 0, 1-bound_height))
					if(c < 0) c = 0		  //Found the first atom.
					if(A.density) c += 1  //Found a dense atom.
				return c  //Return number of dense atoms found, or -1 if there were no atoms found (i.e. the search was at the map's edge).
			BumpBelow() //Find all atoms that are directly (1 pixel) below the src, and bump those atoms.
				for(var/atom/A in obounds(src, 0, -1, 0, 1-bound_height))
					if(A.density && istype(A, /atom/movable))
						Bump(A) //Bump this dense atom.
			MobsBelow()
				var/L = new/list
				for(var/mob/M in obounds(src, 0, -1, 0, 1-bound_height))
					L += M
				return L
		Bump(atom/obstacle)
			switch(dir)
				if(SOUTH) //Touched ground.
					jump = 0		//Reset jump count.
					jump_speed = 0	//Reset jump speed.
					if(client)
						client.north = 0
						client.camera.findIdeals()
				if(NORTH) //Bumped head.
					jump_speed_peaked = 1
					jump_speed = 0	//Reset jump speed.
			if(DensityBelow())
				jump = 0		//Reset jump count.
				jump_speed = 0	//Reset jump speed.
				if(client) client.camera.findIdeals()
			..()