/*
	Gap-Nudge Movement and Default Movement
	By Tyruswoo
*/

client
	var
		diagonal_nudge = FALSE	//FALSE to nudge by mob.step_size on diagonal collisions (fast slide along walls). TRUE to nudge by nudge_size (slow slide).
		f = 0.5					//Min 0 to Max 1. Fraction of player's bound_width or bound_height to look for a gap.
		nudge_size = 1			//Number of pixels to nudge player's mob when finding a gap towards which to move. (Increasing above 1 may cause bugs on narrow gaps.)
	proc
		Default_Northeast() return mob.Move(mob.loc, NORTHEAST, mob.step_x+mob.step_size, mob.step_y+mob.step_size) //Here we replicate BYOND's default movement.
		Default_Northwest() return mob.Move(mob.loc, NORTHWEST, mob.step_x-mob.step_size, mob.step_y+mob.step_size)
		Default_Southeast() return mob.Move(mob.loc, SOUTHEAST, mob.step_x+mob.step_size, mob.step_y-mob.step_size)
		Default_Southwest() return mob.Move(mob.loc, SOUTHWEST, mob.step_x-mob.step_size, mob.step_y-mob.step_size)
		Default_North() return mob.Move(mob.loc, NORTH, mob.step_x, mob.step_y+mob.step_size)
		Default_South() return mob.Move(mob.loc, SOUTH, mob.step_x, mob.step_y-mob.step_size)
		Default_East() return mob.Move(mob.loc, EAST, mob.step_x+mob.step_size, mob.step_y)
		Default_West() return mob.Move(mob.loc, WEST, mob.step_x-mob.step_size, mob.step_y)
		GapNudge_North()	//Player is trying to go northward.
			north = 1					//North was activated. Keep track of this.
			if(east) GapNudge_Northeast()		//If east is also activated, then move Northeast.
			else if(west) GapNudge_Northwest()	//Else if west is also activated, then move Northwest.
			else if(!(Default_North()))			//Else if normal North movement fails, then look for nearby gaps.
				switch(FindGap(NORTH))	//Look for nearby gaps to the north.
					if(EAST) Nudge(NORTH, nudge_size, 0)	//If a northward gap is found nearby to the east, then nudge towards it.
					if(WEST) Nudge(NORTH, -nudge_size, 0)	//If a northward gap is found nearby to the west, then nudge towards it.
		GapNudge_East()
			east = 1
			if(north) GapNudge_Northeast()
			else if(south) GapNudge_Southeast()
			else if(!(Default_East()))
				switch(FindGap(EAST))
					if(NORTH) Nudge(EAST, 0, nudge_size)
					if(SOUTH) Nudge(EAST, 0, -nudge_size)
		GapNudge_South()
			south = 1
			if(east) GapNudge_Southeast()
			else if(west) GapNudge_Southwest()
			else if(!(Default_South()))
				switch(FindGap(SOUTH))
					if(EAST) Nudge(SOUTH, nudge_size, 0)
					if(WEST) Nudge(SOUTH, -nudge_size, 0)
		GapNudge_West()
			west = 1
			if(north) GapNudge_Northwest()
			else if(south) GapNudge_Southwest()
			else if(!(Default_West()))
				switch(FindGap(WEST))
					if(NORTH) Nudge(WEST, 0, nudge_size)
					if(SOUTH) Nudge(WEST, 0, -nudge_size)
		GapNudge_Northeast()
			if(diagonal_nudge)
				if(!(Default_Northeast() || Nudge(NORTHEAST, 0, nudge_size) || Nudge(NORTHEAST, nudge_size, 0))) Default_Northeast()
			else
				if(!(Default_Northeast() || Nudge(NORTHEAST, 0, mob.step_size) || Nudge(NORTHEAST, mob.step_size, 0))) Default_Northeast()
		GapNudge_Northwest()
			if(diagonal_nudge)
				if(!(Default_Northwest() || Nudge(NORTHWEST, 0, nudge_size) || Nudge(NORTHWEST, -nudge_size, 0))) Default_Northwest()
			else
				if(!(Default_Northwest() || Nudge(NORTHWEST, 0, mob.step_size) || Nudge(NORTHWEST, -mob.step_size, 0))) Default_Northwest()
		GapNudge_Southeast()
			if(diagonal_nudge)
				if(!(Default_Southeast() || Nudge(SOUTHEAST, 0, -nudge_size) || Nudge(SOUTHEAST, nudge_size, 0))) Default_Southeast()
			else
				if(!(Default_Southeast() || Nudge(SOUTHEAST, 0, -mob.step_size) || Nudge(SOUTHEAST, mob.step_size, 0))) Default_Southeast()
		GapNudge_Southwest()
			if(diagonal_nudge)
				if(!(Default_Southwest() || Nudge(SOUTHWEST, 0, -nudge_size) || Nudge(SOUTHWEST, -nudge_size, 0))) Default_Southwest()
			else
				if(!(Default_Southwest() || Nudge(SOUTHWEST, 0, -mob.step_size) || Nudge(SOUTHWEST, -mob.step_size, 0))) Default_Southwest()
		GapNudge_NorthReleased()
			north = 0
		GapNudge_EastReleased()
			east = 0
		GapNudge_SouthReleased()
			south = 0
		GapNudge_WestReleased()
			west = 0
		Nudge(d=0, x=0, y=0) //d is direction to face while being nudged. x and y are amounts to nudge, in pixels.
			return mob.Move(mob.loc, d, mob.step_x+x, mob.step_y+y)
		FindGap(d) //d is direction player is facing.
			switch(d)
				if(NORTH)
					if(DensityCount(f,1,-f,-1,0,0,0,1) == 0) return EAST
					else if(DensityCount(0,1,-f,-1,0,0,0,1) == 0) return WEST
				if(EAST)
					if(DensityCount(1,f,-1,-f,0,0,1,0) == 0) return NORTH
					else if(DensityCount(1,0,-1,-f,0,0,1,0) == 0) return SOUTH
				if(SOUTH)
					if(DensityCount(f,0,-f,-1,0,-1,0,1) == 0) return EAST
					else if(DensityCount(0,0,-f,-1,0,-1,0,1) == 0) return WEST
				if(WEST)
					if(DensityCount(0,f,-1,-f,-1,0,1,0) == 0) return NORTH
					else if(DensityCount(0,0,-1,-f,-1,0,1,0) == 0) return SOUTH
		DensityCount(fx,fy,fw=0,fh=0,dx=0,dy=0,dw=0,dh=0) //Fractions used to find offset_x, offset_y, extra_width, & extra_height for obounds proc. Plus dx,dy,dw,dh offsets.
			var/c = -1 //Counts the number of dense atoms found.  -1 if no atoms found (i.e. the search was at the map's edge).
			for(var/atom/A in obounds(mob, mob.bound_width*fx+dx, mob.bound_height*fy+dy, mob.bound_width*fw+dw, mob.bound_height*fh+dh))
				if(c < 0) c = 0		  //Found the first atom.
				if(A.density) c += 1  //Found a dense atom.
			return c  //Return number of dense atoms found, or -1 if there were no atoms found (i.e. the search was at the map's edge).